
# q-expansion of newform 1914.2.a.d, downloaded from the LMFDB on 09 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1914
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[-1], [-1], [1], [3], [-1], [-4], [5], [-7], [-6], [-1], [-2], [-5], [1], [-9], [-3], [10], [7], [-2], [6], [-12], [-10], [10], [12], [6], [-18], [-8], [11], [-15], [6], [13], [-8], [-20], [2], [-10], [-3], [-1], [3], [-11], [-12], [-7], [-12], [-18], [3], [24], [11], [20], [-15], [16], [-15], [-23], [22], [-12], [7], [20], [28], [-5], [12], [8], [-12], [6], [-8], [2], [4], [-23], [-33], [28], [-35], [2], [3], [-14], [4], [-25], [-22], [16], [-4], [30], [12], [22], [16], [-10], [25], [26], [-22], [34], [9], [-28], [27], [37], [20], [-8], [10], [-24], [7], [-12], [-6], [-11], [7], [-42], [20], [-19], [20], [-21], [14], [-37], [-22], [20], [-35], [28], [-4], [-46], [-18], [-16], [21], [-35], [27], [-15], [46], [26], [-30], [10], [42], [-11], [-12], [27], [40], [-29], [-14], [22], [38], [-10], [-4], [-11], [-4], [-7], [20], [30], [-8], [14], [-24], [26], [-42], [-54], [26], [-42], [7], [-3], [11], [-6], [9], [-30], [-8], [-30], [30], [28], [-44], [-16], [-29], [42], [-11], [-46], [24], [26], [-50], [26], [-24], [4], [19], [-5], [-9], [38], [-24], [-32], [22], [-22], [23], [-38], [2], [-18], [-4], [36], [60], [6], [-38], [30], [-22], [22], [53], [-49], [-20], [34], [-49], [18], [28], [42], [36], [-18], [32], [-44], [10], [36], [62], [33], [21], [-26], [-2], [-19], [6], [47], [-50], [-44], [28], [25], [-2], [-24], [-23], [26], [-59], [-10], [66], [-20], [-14], [64], [-45], [-60], [-48], [-43], [51], [64], [56], [-68], [58], [62], [-16], [1], [-46], [-27], [60], [52], [67], [-9], [-6], [-14], [-72], [71], [40], [-70], [40], [60], [8], [-36], [70], [-52], [-3], [22], [56], [9], [-75], [7], [38], [26], [60], [-6], [-38], [-30], [-43], [21], [66], [35], [33], [43], [68], [-24], [-25], [-66], [44], [-22], [40], [-54], [-42], [-15], [42], [3], [-24], [-62], [-57], [-25], [22], [-48], [-35], [-30], [-38], [-4], [24], [42], [-34], [49], [-8], [-78], [37], [-41], [4], [63], [44], [14], [12], [-79], [-44], [-2], [18], [86], [-24], [-6], [-6], [-14], [-46], [-57], [-44], [80], [-77], [-67], [35], [-69], [-37], [-2], [12], [70], [-80], [-26], [64], [-37], [61], [31], [-5], [-10], [4], [-91], [-30], [-6], [-2], [57], [55], [48], [26], [-25], [-3], [-86], [23], [-89], [0], [28], [4], [-54], [-36], [-28], [-1], [-51], [64], [-85], [-6], [-42], [6], [30], [-77], [-84], [-39], [-36], [30], [57], [-40], [12], [24], [-78], [-30], [-91], [40], [86], [-49], [38], [0], [-37], [-72], [-68], [96], [-15], [14], [66], [-50], [-58], [-39], [40], [-98], [41], [72], [-69], [87], [-69], [21], [-47], [-48], [90], [-36], [26], [-24], [88], [-17], [-76], [-14], [-33], [84], [-57], [-26], [-45], [89], [-22], [28], [-44], [85], [87], [-46], [-27], [-51], [58], [-13], [-90], [-70], [-6], [-7], [-69], [70], [57]]
